#ifndef __TEXT_DIFF_DIFF_HELPER_HPP__
#define __TEXT_DIFF_DIFF_HELPER_HPP__

#include <string>
#include <vector>

std::string diff_sequence(
        const std::vector<std::string> &,
        const std::vector<std::string> &
        );

#endif
